/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - March 20, 2000.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2000
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
   - Contributor(s):
   -  Henk-Johan van Rantwijk <bugs4hj@netscape.net> (Original Author)
   -
   - Alternatively, the contents of this file may be used under the
   - terms of the GNU General Public License Version 2 or later (the
   - "GPL"), in which case the provisions of the GPL are applicable
   - instead of those above.  If you wish to allow use of your
   - version of this file only under the terms of the GPL and not to
   - allow others to use your version of this file under the MPL,
   - indicate your decision by deleting the provisions above and
   - replace them with the notice and other provisions required by
   - the GPL.  If you do not delete the provisions above, a recipient
   - may use your version of this file under either the MPL or the
   - GPL.
   -
*/

var multiviewsBundle = "chrome://multiviews/locale/multiviews.properties";
var mzToolbox = null;

const gbToolboxVisibilityPrefListener =
{
  domain: "multizilla.integrated.toolbar",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var isVisible = pref.getBoolPref(prefName);
    mzToolbox.mzToolboxSetVisibility(isVisible);
  }
};
const gTabMinWidthPrefListener =
{
  domain: "multizilla.tabs.mintabwidth",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabWidth = pref.getIntPref(prefName);
    gBrowser.setTabWidthTo("minwidth", tabWidth);
  }
};
const gTabWidthPrefListener =
{
  domain: "multizilla.tabs.tabwidth",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabWidth = pref.getIntPref(prefName);
    gBrowser.setTabWidthTo("maxwidth", tabWidth);
  }
};
const gTabLEDstatePrefListener =
{
  domain: "multizilla.tabs.status-leds",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabLEDstate = pref.getBoolPref(prefName);
    gBrowser.setTabLEDstateTo(tabLEDstate);
  }
};
const gTabLabelPrefListener =
{
  domain: "multizilla.tabs.title",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabLabel = pref.getBoolPref(prefName);
    gBrowser.setTabLabelsTo(tabLabel);
  }
};
const gTabLabelCropPrefListener =
{
  domain: "multizilla.tabs.cropping",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;
    gBrowser.setLabelCropping();
  }
};
const gTabSwitchPrefListener =
{
  domain: "multizilla.tabs.ontop",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var topTabs = pref.getBoolPref(prefName);
    if (topTabs)
      gBrowser.activateTopTabs()
    else gBrowser.activateBottomTabs();
  }
};
const gTabCanClosePrefListener =
{
  domain: "multizilla.tabs.iconclose",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabCanClose = pref.getBoolPref(prefName);
    gBrowser.setTabCanClose(tabCanClose);
  }
};
const gTBButtonPrefListener =
{
  domain: "multizilla.showbutton",
  observe: function(subject, topic, prefName)   {
    if (topic != "nsPref:changed")
      return;
    var buttonName = prefName.substr(this.domain.length+1);
    var buttonId = buttonName + "-tbbutton";
    var button = document.getElementById(buttonId);
    var show = pref.getBoolPref(prefName);

    if (show) 
      button.removeAttribute("hidden");
    else button.setAttribute("hidden", "true");
  }
};
const gOrdinancePrefListener =
{
  domain: "multizilla.tabs.open-order",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var ordinance = pref.getIntPref(prefName);
    gBrowser.mTabOrdinance = ordinance;
  }
};
const gtabFocusOrderPrefListener =
{
  domain: "multizilla.tabs.focus-order",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabFocusOrder = pref.getIntPref(prefName);
    gBrowser.mTabFocus = tabFocusOrder;
  }
};
const gtabbarCloseButtonPrefListener =
{
  domain: "multizilla.tabbar.closebutton",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var closeButton = pref.getBoolPref(prefName);
    gBrowser.mTabContainer.setAttribute("displayclose", closeButton ? "true" : "false");
  }
};
const gtabbarNewButtonPrefListener =
{
  domain: "multizilla.tabbar.newbutton",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var newButton = pref.getBoolPref(prefName);
    gBrowser.mTabContainer.setAttribute("displaynew", newButton ? "true" : "false");
  }
};
const gTabSiteIconsPrefListener =
{
  domain: "browser.chrome.site_icons",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    gBrowser.updateDisplayTabIcons();
  }
};
const gTabFaviconsPrefListener =
{
  domain: "browser.chrome.favicons",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    gBrowser.updateDisplayTabIcons();
  }
};
const gTabAdvancedOpenTabPrefListener =
{
  domains: [ "multizilla.newtabfor.native-applications",
             "advanced.system.supportDDEExec" ],

  observe: function(subject, topic, aPrefName) {
    if (topic != "nsPref:changed")
      return;

    if (aPrefName == "multizilla.newtabfor.native-applications")
      pref.setBoolPref(this.domains[1], !pref.getBoolPref(aPrefName));
    else if (aPrefName == "advanced.system.supportDDEExec")
      pref.setBoolPref(this.domains[0], !pref.getBoolPref(aPrefName));
  }
};
const gTabSessionStartupPrefListener =
{
  domains: [ "browser.startup.page",
             "multizilla.browsersession.skip.navigator.startup",
             "multizilla.tabs.startgroup" ],

  observe: function(subject, topic, aPrefName) {
    if (topic != "nsPref:changed")
      return;

    var prefValue;

    if (aPrefName == "multizilla.browsersession.skip.navigator.startup") {
      prefValue = pref.getBoolPref(aPrefName);
      if (prefValue)
        pref.setIntPref(this.domains[0], 0);
    }
    else if (aPrefName == "browser.startup.page") {
      prefValue = pref.getIntPref(aPrefName);
      if (prefValue > 0)
        pref.setBoolPref(this.domains[1], false);
    }
    else if (aPrefName == "multizilla.tabs.startgroup") {
      prefValue = pref.getBoolPref(aPrefName);
      if (prefValue)
        pref.setBoolPref(this.domains[1], true);
    }
  }
};
////////////////////////////////////////////////////////////////////////////////
function confirmCloseWS(aConfirm)
{
  var confirmPref = readMyPref("multizilla.confirm.quit", "int", 0);
  var closeBundle = srGetStrBundle("chrome://multiviews/locale/multiviews.properties");
  var aHeader = closeBundle.GetStringFromName("closeSession");
  var aQuestion = closeBundle.GetStringFromName("closeSessionQuestion");
  var tabs = gBrowser.mPanelContainer.childNodes.length;
  /* ::::
     1  = File Menu 'Close Window' & Ctrl-Shift-W
     2  = File Menu 'Exit' & Ctrl-Q
     4  = Click on [X] on Window
     8  = JavaScript close()
     16 = no confirmation for single tab
  :::: */
  if (tabs == 1 && confirmPref & 16) {
    checkGroupSaving();
    removeIdentifier();
    return true;
  }
  else if (aConfirm == 8 && document.documentElement.hasAttribute("chromehidden")) {
    removeIdentifier();
    return true;
  }
  else if (confirmPref && (confirmPref & aConfirm) == aConfirm) {
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                        getService(Components.interfaces.nsIPromptService);

    if (aConfirm & 1) {
      aHeader = closeBundle.GetStringFromName("closeWindow");
      aQuestion = closeBundle.GetStringFromName("closeWindowQuestion");
    }
    if (promptService.confirm(window, aHeader, aQuestion)) {
      checkGroupSaving();
      removeIdentifier();
      return true;
    }
    else
      return false;
  }
  checkGroupSaving();
  removeIdentifier(); 
  return true;
}
////////////////////////////////////////////////////////////////////////////////
function multiZillaInit()
{
  addPrefListener(gTabMinWidthPrefListener);
  addPrefListener(gTabWidthPrefListener);
  addPrefListener(gTabLEDstatePrefListener);
  addPrefListener(gTabLabelPrefListener);
  addPrefListener(gTabLabelCropPrefListener);
  addPrefListener(gTabSwitchPrefListener);
  addPrefListener(gTBButtonPrefListener);
  addPrefListener(gTabCanClosePrefListener);
  addPrefListener(gbToolboxVisibilityPrefListener);
  addPrefListener(gOrdinancePrefListener);
  addPrefListener(gtabFocusOrderPrefListener);
  addPrefListener(gtabbarNewButtonPrefListener);
  addPrefListener(gtabbarCloseButtonPrefListener);
  addPrefListener(gTabSiteIconsPrefListener);
  addPrefListener(gTabFaviconsPrefListener);
  addPrefListener(gTabAdvancedOpenTabPrefListener);
  addPrefListener(gTabSessionStartupPrefListener);
  initPreferences();
  // Initialize "multizilla.rdf"
  initializeRDFStorageFile(); // located in quickPrefsMenu.js (for QPM stand-alone)
}
////////////////////////////////////////////////////////////////////////////////
// this function sets ID's to the Go Menu and replaces the popup
// I know this is a dirty way to do it, but we need to ;)
////////////////////////////////////////////////////////////////////////////////
function addGoMenuIDs()
{
  var mainMenu = document.getElementById("main-menubar");
  var goMenu = mainMenu.childNodes[3];
  goMenu.setAttribute("id", "menu_Go");
  goMenu.firstChild.setAttribute("id", "menu_GoPopup");
  var goMenuPopup = document.getElementById("newMenu_GoPopup");
  // We can't use replaceChild because that is broken in mozilla!!!
  // goMenu.replaceChild(goMenuPopup, goMenu.firstChild);
  goMenu.insertBefore(goMenuPopup, goMenu.firstChild);
  goMenu.removeChild(goMenu.lastChild);
}
////////////////////////////////////////////////////////////////////////////////
function initBookmarkTemplates()
{
  initializeBookmarksTemplate();
  initializePersonalToolbarTemplate();

  // select modified template for the Bookmarks menu
  var bookmarksMenu = document.getElementById("BookmarksMenu");
  bookmarksMenu.setAttribute("template", "BookmarksMenuTemplate");
  // select modified template for the Bookmarks Button
  var bookmarkButton = document.getElementById("bookmarks-button");
  bookmarkButton.setAttribute("template", "BookmarksButtonTemplate");
  // select modified template for the Bookmarks Overflow Menu
  var bookmarksChevron = document.getElementById("bookmarks-chevron");
  bookmarksChevron.setAttribute("template", "BookmarksButtonTemplate");
  // replace default PT template with our modified version
  var originalPTBox = document.getElementById("bookmarks-ptf");
  var newPTBox = document.getElementById("NEW-NC:PersonalToolbarFolder");
  var clonedTemplate = newPTBox.childNodes[1].cloneNode(true);
  originalPTBox.replaceChild(clonedTemplate, originalPTBox.childNodes[1]);

  if (!/Mac/.test(navigator.platform)) { // Modify Main Bookmarks menu
    // bookmarksMenu.removeAttribute("oncommand"); XXX: see bug 3324
    // bookmarksMenu.setAttribute("onclick", "OpenBookmarkURL(event.target, this.database, event)");
  }
  // cleanup: remove dummy bookmarks menu item
  var dummyBookmarkPopup = document.getElementById("dummyBookmarkPopup");
  dummyBookmarkPopup.removeChild(newPTBox);

  // theme support: added for Mac(OSX)Zilla support
  bookmarkButton.firstChild.setAttribute("class", "bookmarks-menupopup");
  bookmarksChevron.firstChild.setAttribute("class", "bookmarks-menupopup");
}
////////////////////////////////////////////////////////////////////////////////
function initInstalledToolbox()
{
  // GoogleBox support: added for MultiZilla's GoogleBox
  mzToolbox = document.getElementById("MultiZillaToolbar");
  var toolboxInstalled = readMyPref("multizilla.integrated.toolbar", "bool", false);
  mzToolbox.mzToolboxSetVisibility(toolboxInstalled);
  var showInternalToolbox = document.getElementById("cmd_viewIntegratedToolBox").getAttribute("checked");

  if (toolboxInstalled && showInternalToolbox == "true")
    document.getElementById("internalToolBox").removeAttribute("hidden");
  if (mzToolbox)
    mzToolbox.updateButtons("multizilla.showbutton.");
}
////////////////////////////////////////////////////////////////////////////////
function initOrdinals()
{
  // Initialize ordinals
  var nodeList = document.getAnonymousNodes(getBrowser().mTabContainer);
  var index = /Mac/.test(navigator.platform) ? 1 : 0;

  for (var i = (0+index); i < nodeList.length; i++) {
    nodeList[i].ordinal = i;
    if (nodeList[i].localName == "hbox") {
      nodeList[i].firstChild.ordinal = 0;
      nodeList[i].lastChild.ordinal = 9999999;
    }
  }
  nodeList[nodeList.length-(1+index)].ordinal = 9999999;
  gBrowser.mTabContainer.advanceSelectedTab = advanceSelectedTab;
}
////////////////////////////////////////////////////////////////////////////////
function initEnhancedPopupBlocker()
{
  // Remove default popup pref listener, because it fails to work properly!
  removePrefListener(gPopupPrefListener);
  // Use a 'heavily' modified copy of it
  addPrefListener(mzPopupPrefListener);
  /****
  * We need to get notified of blocked popup windows so we replace the 
  * original eventlistener for 'DOMPopupBlocked' with our (this) event 
  * listener and call 'onPopupBlocked' (original function) when we're done.
  */
  addEventListener("DOMPopupBlocked", mzOnPopupBlocked, false);
  removeEventListener("DOMPopupBlocked", onPopupBlocked, false);
}

var firstBrowserWindow = null;
////////////////////////////////////////////////////////////////////////////////
function isFirstWindow(aType)
{
  var defaultType = "navigator:browser";

  if (!aType) // default check for browser window
    aType = defaultType;

  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var topWindow = windowManagerInterface.getMostRecentWindow(null);
  var enumerator = windowManagerInterface.getEnumerator(aType);
  var firstWindow = enumerator.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
  var windowType = topWindow.document.documentElement.getAttribute("windowtype")
  // Initialize 'firstBrowserWindow' the first time only!
  if (firstBrowserWindow == null && windowType == defaultType)
    firstBrowserWindow = firstWindow; // this is used in mzWindowInit()

  if (firstWindow != topWindow)
    return false;
  return true;
}
////////////////////////////////////////////////////////////////////////////////
function checkGroupSaving()
{
  var shouldSave = readMyPref("multizilla.tabs.saveonexit", "bool", false);
  var tabCount = getBrowser().mPanelContainer.childNodes.length;

  if (!shouldSave)
    return;
  else saveBrowserSession();
}
////////////////////////////////////////////////////////////////////////////////
function useFirstPage()
{
 var uriToLoad = null;
  if ("arguments" in window && window.arguments.length >= 1 && window.arguments[0])
    uriToLoad = window.arguments[0];
  if (readMyPref("multizilla.tabs.blank-tab-first", "bool", true)) {
    if ((readMyPref("browser.startup.page", "int", 1) == 1 && 
        getHomePage() == "about:blank" || getHomePage() == "") || uriToLoad == "about:blank" || 
        readMyPref("browser.startup.page", "int", 0) == 3 && uriToLoad == "about:blank")
      return true;
  }
  return false;
}
////////////////////////////////////////////////////////////////////////////////
// This function modifies the default mozilla startup.
////////////////////////////////////////////////////////////////////////////////
function mzWindowInit()
{
  // dump("\nIN - window.opener: "+window.opener);
  // dump("\nwindow.arguments: "+('arguments' in window ? true : false));
  var newTab = null;
  var externalLinks = readMyPref("multizilla.newtabfor.external-links", "bool", true);
  var nativeApplications = readMyPref("multizilla.newtabfor.native-applications", "bool", true);

  if ((externalLinks || nativeApplications) && 'arguments' in window && window.opener == null) {
    var url = window.arguments[0]; // Single file only for now!
    // dump("\nurl: " +url);
    if (url != "" && url != null) { //  && url != "about:blank") {
      // dump("\nValid URL");
      if (!isFirstWindow() && firstBrowserWindow != null) {
        // dump("\nIs Not First Window");
        var tabbrowser = firstBrowserWindow.getBrowser();
        firstBrowserWindow.focus();
        if (readMyPref("multizilla.tabs.blank-tab-first", "bool", true)) {
          newTab = checkForBlankTab(tabbrowser);
          if (newTab != null)
            newTab.loadURI(url);
        }
        if (newTab == null) { // Add new tab with URL. Pref not set or no blank tab available
          newTab = tabbrowser.addTab(url);
          tabbrowser.selectedTab = newTab; // Don't we need a pref setting for this?
        }
        window.document.documentElement.setAttribute("onunload", "");
        window.close();
        return false;
      }
      else { 
        /**
          * This is, well it should be, the first navigator:browser window. 
          * Do we need to restore a previously saved browser session?
          */
        if (readMyPref("multizilla.tabs.startgroup", "bool", true)) {
          var startup = pref.getIntPref("browser.startup.page");
          // Browser sessions will only be restored in limited cases
          if (startup == 0 && url == "about:blank")
            getBrowser().mRestoreBrowserSession = true;
          else if (startup == 1 && url == getHomePage())
            getBrowser().mRestoreBrowserSession = true;
          else if (startup == 2) {
            var history = Components.classes["@mozilla.org/browser/global-history;1"].getService(Components.interfaces.nsIBrowserHistory);
            if (url == history.lastPageVisited)
              getBrowser().mRestoreBrowserSession = true;
          }
        }
      }
    }
  }
  else {
    /* var url = getBrowser().mCurrentBrowser.currentURI.spec;
    dump("\nELSE: "+url);
    try {
      var event = document.createEvent("Events");
      dump("\nevent: "+event);
      event.initEvent("PopupWindow", false, true);
      var foopy = document.getElementById("main-window");
      dump("\nfoopy: "+foopy);
      foopy.dispatchEvent(event);
    } catch(e) { dump(e) } */
  }
  Startup();
  setIdentifier();
  if (!getBrowser().mInit)
    getBrowser().enterTabbedMode();
}